package java.security.cert;

import java.io.Serializable;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.security.GeneralSecurityException;

public class CertPathValidatorException extends GeneralSecurityException
{
    private static final long serialVersionUID = -3083180014971893139L;
    private int index;
    private CertPath certPath;
    private CertPathValidatorException.Reason reason;
    
    public CertPathValidatorException() {
        this(null, null);
    }
    
    public CertPathValidatorException(String s) {
        this(s, null);
    }
    
    public CertPathValidatorException(Throwable t) {
        this((t == null) ? null : t.toString(), t);
    }
    
    public CertPathValidatorException(String s, Throwable t) {
        this(s, t, null, -1);
    }
    
    public CertPathValidatorException(String s, Throwable t, CertPath certPath, int n) {
        this(s, t, certPath, n, CertPathValidatorException.BasicReason.UNSPECIFIED);
    }
    
    public CertPathValidatorException(String message, Throwable cause, CertPath certPath, int index, CertPathValidatorException.Reason reason) {
        super(message, cause);
        this.index = -1;
        this.reason = CertPathValidatorException.BasicReason.UNSPECIFIED;
        if (certPath == null && index != -1) {
            throw new IllegalArgumentException();
        }
        if (index < -1 || (certPath != null && index >= certPath.getCertificates().size())) {
            throw new IndexOutOfBoundsException();
        }
        if (reason == null) {
            throw new NullPointerException("reason can't be null");
        }
        this.certPath = certPath;
        this.index = index;
        this.reason = reason;
        return;
    }
    
    public CertPath getCertPath() {
        return this.certPath;
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public CertPathValidatorException.Reason getReason() {
        return this.reason;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        if (this.reason == null) {
            this.reason = CertPathValidatorException.BasicReason.UNSPECIFIED;
        }
        if (this.certPath == null && this.index != -1) {
            throw new InvalidObjectException("certpath is null and index != -1");
        }
        if (this.index < -1 || (this.certPath != null && this.index >= this.certPath.getCertificates().size())) {
            throw new InvalidObjectException("index out of range");
        }
        return;
    }
    
    public enum BasicReason implements CertPathValidatorException.Reason
    {
        UNSPECIFIED, 
        EXPIRED, 
        NOT_YET_VALID, 
        REVOKED, 
        UNDETERMINED_REVOCATION_STATUS, 
        INVALID_SIGNATURE, 
        ALGORITHM_CONSTRAINED;
    }
    
    public interface Reason extends Serializable
    {
    }
}
